@@ -28,6 +28,8 @@ use log::{debug, info, trace, warn};
 use db::DB;
 use runctx::RunContext;
 
+use terminal_size::{terminal_size, Height, Width};
+
 use err::Error;
 
 type CmdProc = fn(&str, &[String]) -> Result<(), Error>;
@@ -677,11 +679,82 @@ fn proc_list_tags(cmd: &str, args: &[String]) -> Result<(), Error> {
   //
   let lst = db.get_tag_list()?;
 
-  //
-  // Dump table
-  //
-  for (id, title) in lst {
-    println!("{:4} {}", id, title);
+  let slist: Vec<(String, String)> = lst
+    .iter()
+    .map(|(id, title)| (id.to_string(), title.clone()))
+    .collect();
+
+  let mut max_id_len = 0;
+  let mut max_title_len = 0;
+  for (ids, title) in &slist {
+    if ids.len() > max_id_len {
+      max_id_len = ids.len();
+    }
+    if title.len() > max_title_len {
+      max_title_len = title.len();
+    }
+  }
+
+  let size = terminal_size();
+  let dump_list = if let Some((Width(w), Height(h))) = size {
+    if slist.len() > h as usize / 2 {
+      //
+      // Dump table
+      //
+
+      // single entry width  (longest id + 1 space + longest title + 2 spaces
+      // to separate columns).
+      // ToDo: This wastes some space unecessarily -- fix this.
+      let swidth = max_id_len + 1 + max_title_len + 2;
+
+      // Calculate the number of columns to use
+      let ncols = w as usize / swidth;
+
+      // Calculate rows
+      let nrows = if slist.len() % ncols == 0 {
+        slist.len() / ncols
+      } else {
+        slist.len() / ncols + 1
+      };
+
+      for row in 0..nrows {
+        for col in 0..ncols {
+          let idx = nrows * col + row;
+          if idx < slist.len() {
+            print!(
+              "{ids:>idwidth$} {title:titlewidth$}",
+              ids = slist[idx].0,
+              title = slist[idx].1,
+              idwidth = max_id_len,
+              titlewidth = max_title_len
+            );
+
+            if col < ncols - 1 {
+              print!("  ");
+            }
+          }
+        }
+        println!("");
+      }
+      false
+    } else {
+      true
+    }
+  } else {
+    true
+  };
+
+  if dump_list {
+    //
+    // Dump list if table wasn't dumped
+    //
+    for (ids, title) in lst {
+      println!(
+        "{ids:>idwidth$} {title:titlewidth$}",
+        idwidth = max_id_len,
+        titlewidth = max_title_len
+      );
+    }
   }
 
   Ok(())
